%  Copyright 2013 Kwang In Kim, Max-Planck-Institut for Informatics
%  Please refer to license.txt for sanctioned usage.
%
% [L] = ConstructCANormalLap(Nodes, NNIdx, SigmaSq, TanDim)
% INPUT:
% 'Nodes': (N times d) data matrix; 'N': number of data points, 'd': 
%           dimensionality of data.
% 'NNIdx': (N times k) matrix containing indices of k-nearest neighbors for
%           each data point (This matrix can be obtained using GetKNN).
% 'SigmaSq': the 'usual' SigmaSq parameter for the similarity matrix used in building graph Laplacian.
% 'TanDim': an integer specifying the dimensionality of the manifold
%             (for SSR we recommend to do cross-validation over this parameter if
%             the dimensionality of the manifold is unknown).
%
% OUTPUT:
% 'L': (N times N) matrix - Curvature-aware graph Laplacian matrix L as described in the paper.

function [L] = ConstructCANormalLap(Nodes, NNIdx, SigmaSq, TanDim)

[NumNodes,~] = size(Nodes);

[L] = ConstructCAW(Nodes, NNIdx, SigmaSq, TanDim);
D = diag(sum(L,1));
L = D-L;

%%% normalize the Laplacian
    for i=1:NumNodes
        if D(i,i)~=0
            D(i,i) = 1/sqrt(D(i,i));
        end
    end
    L = D'*L*D;
    L = (L'+L)/2;
%%% normalize the Laplacian
