%  Copyright 2013 Kwang In Kim, Max-Planck-Institut for Informatics
%  Please refer to license.txt for sanctioned usage.
%
function [TCurva] = ConstructLocalTCoord(Nodes, Base, BaseIndexInG, LatDim)

TRUNCLIMIT = 0.1e-9;

[NumNodes,TDim] = size(Nodes);
LatNodes = Nodes(:,1:LatDim);
%AmbNodes = Nodes(:,LatDim+1:end);
LatBase = Base(:,1:LatDim);
%AmbBase = Base(:,LatDim+1:end);

[X] = ComputeX(LatNodes);
% [XBase] = ComputeX(LatBase);
X = X(:,1:end-1);
% XBase = XBase(:,1:end-1);

XInv = pinv(X);

Vec = zeros(NumNodes, LatDim);
TCurva = zeros(NumNodes,1);
for i=1:NumNodes
    Vec(i,:) = LatNodes(i,:)-LatBase;
    Norm = norm(Vec(i,:));
    if Norm > TRUNCLIMIT
        Vec(i,:) = Vec(i,:)/Norm;
    end
end

%%% calculate Hessian for each AmbCoordinate
    THessMat = zeros(LatDim,LatDim);
    for i=LatDim+1:TDim
        HessMat = zeros(LatDim,LatDim);
        Y = Nodes(:,i);
        YC = Y-(Y(BaseIndexInG));
        A = XInv*YC;

%         plot3(Nodes(:,1),Nodes(:,2),YC,'.');
        RIdx = 1;
        for j=1:LatDim
            for k=j:LatDim
                if j==k
                    HessMat(j,k) = 2*A(RIdx);
                else
                    HessMat(j,k) = A(RIdx);
                end
                RIdx = RIdx+1;
            end
        end
        for j=1:LatDim
            for k=j:LatDim
                HessMat(k,j) = HessMat(j,k);
            end
        end
        %%% makes the individual Hessian positive definite
            [EVec,EVal] = eig(HessMat);
            HessMat = EVec*abs(EVal)*EVec';
        %%% makes the individual Hessian positive definite
        THessMat = THessMat+HessMat;
    end
%%% calculate Hessian for each AmbCoordinate

for j=1:NumNodes
    S = Vec(j,:)*THessMat;
    TCurva(j) = S*S';
end
% %%% calculate Hessian for each AmbCoordinate
%     for i=LatDim+1:TDim
%         %%% Estimate principal curvatures
%             HessMat = zeros(LatDim,LatDim);
%             Y = Nodes(:,i);
%             YC = Y-(Y(BaseIndexInG));
%             A = XInv*YC;
% 
%     %         plot3(Nodes(:,1),Nodes(:,2),YC,'.');
%             RIdx = 1;
%             for j=1:LatDim
%                 for k=j:LatDim
%                     if j==k
%                         HessMat(j,k) = 2*A(RIdx);
%                     else
%                         HessMat(j,k) = A(RIdx);
%                     end
%                     RIdx = RIdx+1;
%                 end
%             end
%             for j=1:LatDim
%                 for k=j:LatDim
%                     HessMat(k,j) = HessMat(j,k);
%                 end
%             end
%             [EVec,EVal] = eig(HessMat);
%             EVal = abs(diag(EVal));
%         %%% Estimate principal curvatures
% %         %%% for trace
% %             EVal = [10;0];
% %             EVec = [0.7071,0.7071;0.7071,-0.7071];
% %         %%% for trace
%         %%% calculate total curvatures
%             for j=1:NumNodes
%                 Proj = abs(Vec(j,:)*EVec);
%                 TCurva(j) = TCurva(j)+Proj*EVal;
%             end
%         %%% calculate total curvatures
%     end
% %%% calculate Hessian for each AmbCoordinate
